// A varibale so we know if we're resizing or not
var isResizing = false;

// Variables to hold initial mouse position
var oldhOffset = 0;
var oldvOffset = 0;

// A global varibale to store window handles of widgets
var widgetHandles = new Array();

// The first thing we need to do is update the widget according to preferences
updateWidgetDisplay();

function captureWindow(handle)
{
	// Get the elements required to capture a window successfully
	var winHandle = handle; // The handle of the window to capture
	var imgFormat = preferences.imageFormat.value; // The format to save the image in
	
	strFilename = saveAs(imgFormat);
	
	if (imgFormat == "JPG")
	{
		var imgQuality = preferences.jpegQuality.value;
		var strCommand = "./ScreenShooter.exe -w " + "-" + imgFormat + " " + winHandle + " " + imgQuality + " \"" + resolvePath(strFilename) + "\"";	
	} else{
		var strCommand = "./ScreenShooter.exe -w " + "-" + imgFormat + " " + winHandle + " \"" + resolvePath(strFilename) + "\"";
	}
	var result = runCommand(strCommand);
	
	// Check if the screenshot was taken successfully and print result to debugger
	if (result == "1")
	{
		// Ask if the user would like to view the screenshot
		var answer = alert("Screenshot taken successfully! Would you like to view it now?", "Yes", "No");
		
		// Check their answer and act accordingly
		if (answer == 1)
		{
			openScreenshot(strFilename);
		}
	} else{
		alert("Screenshot not taken! Error returned:\n\n" + result);
	}
}

function getWindowsList()
{
	// Run the .NET app to generate a list of open windows
	var result = runCommand("./WindowManager.exe -l");
	// Parse the result from the command line
	var windowList = result.split("\n");
	
	// Loop through the returned list of windows
	for (i = 0; i < windowList.length; i++)
	{
		// Separate the results of the window list (name, handle)
		var item = windowList[i].split(",");
		// If the window doesn't have a title we aren't interested in it
		if (item[0] == "")
		{
			// Set it to undefined
			windowList[i] = undefined;
		}
	}
	// Send the results back to the caller as an array
	return windowList;
}

function showWindowsListMenu()
{
	// Print up a status telling the user what we are doing
	// The .NET app can take a few moments to start on the first run
		
	// Generate a list of windows that are open
	var windowList = getWindowsList();
	
	// Create a new menu of open windows
	var menuItems = Array();
	
	// Loop through the menu list
	for (i = 0; i < windowList.length; i++)
	{
		if (windowList[i] != undefined)
		{
			var item = windowList[i].split(",");
			menuItems[i] = new MenuItem();
			// If the window title is long we need to truncate it
			if (item[0].length > 50)
			{
				// Truncate long window names to keep the menu reasonably sized
				menuItems[i].title = item[0].substring(0, 50) + "...";
			} else{
				// Window title not long so jsut display it
				menuItems[i].title = item[0];
			}
			// Assign the capture window function to the item
			menuItems[i].onSelect = "captureWindow(" + item[1] + ");";
		}
	}
	
	// Display the menu to the user	
	popupMenu(menuItems, windowButton.hOffset, windowButton.vOffset + windowButton.height);
}

function updateWidgetDisplay()
{	
	// Set the image type drop down to the current setting
	var strImageType = preferences.imageFormat.value;
	imageTypePopupText.data = strImageType;
	
	// Set the hotkey properties
	hkFocus.modifier = preferences.hotkeyFocusWidget.value.substr(0, preferences.hotkeyFocusWidget.value.lastIndexOf("+"));
	hkFocus.key = preferences.hotkeyFocusWidget.value.substr(preferences.hotkeyFocusWidget.value.lastIndexOf("+") + 1);
	
	hkCapture.modifier = preferences.hotkeyCaptureDesktop.value.substr(0, preferences.hotkeyCaptureDesktop.value.lastIndexOf("+"));
	hkCapture.key = preferences.hotkeyCaptureDesktop.value.substr(preferences.hotkeyCaptureDesktop.value.lastIndexOf("+") + 1);
}

function displayImageFormatPopup()
{
	// We need to create and then show a menu of possible image types
	var items = Array();
	
	// Create the menu items
	items[0] = new MenuItem();
	items[0].title = "JPG";
	items[0].onSelect = "imageFormatSelected('JPG');";
	
	items[1] = new MenuItem();
	items[1].title = "BMP";
	items[1].onSelect = "imageFormatSelected('BMP');";
	
	items[2] = new MenuItem();
	items[2].title = "GIF";
	items[2].onSelect = "imageFormatSelected('GIF');";
	
	items[3] = new MenuItem();
	items[3].title = "TIF";
	items[3].onSelect = "imageFormatSelected('TIF');";
	
	items[4] = new MenuItem();
	items[4].title = "PNG";
	items[4].onSelect = "imageFormatSelected('PNG');";
	
	// Display the menu beneath the popup box
	popupMenu(items, imageTypePopupBox.hOffset, imageTypePopupBox.vOffset + imageTypePopupBox.height);
}

function imageFormatSelected(format)
{
	// Act according to the type of image selected
	switch(format)
	{
		case "JPG":
			preferences.imageFormat.value = "JPG";
			updateWidgetDisplay();
			break;
		case "BMP":
			preferences.imageFormat.value = "BMP";
			updateWidgetDisplay();	
			break;
		case "GIF":
			preferences.imageFormat.value = "GIF";
			updateWidgetDisplay();
			break;
		case "TIF":
			preferences.imageFormat.value = "TIF";
			updateWidgetDisplay();
			break;
		case "PNG":
			preferences.imageFormat.value = "PNG";
			updateWidgetDisplay();
			break;
	}
}

function resizeViewFinder()
{
	if (isResizing)
	{
		var newhOffset = (system.event.hOffset - oldhOffset) + 10;
		var newvOffset = (system.event.vOffset - oldvOffset) + 10;
		
		viewFinder.width = newhOffset;
		viewFinder.height = newvOffset;		
		windowBackground.width = newhOffset;
		windowBackground.height = newvOffset;
		
		gripHandle.hOffset = newhOffset - 10;
		gripHandle.vOffset = newvOffset - 10;
		closeButton.hOffset = newhOffset - 10;
	}
}

function captureRectangle()
{
	// This function will capture the rectangle of the viewfinder
	// whatever is under the viewfinder will appear in the screenshot
	
	// Get the properties from the viewfinder
	var x = viewFinder.hOffset;
	var y = viewFinder.vOffset;
	var width = viewFinder.width;
	var height = viewFinder.height;
	
	// Hide the viewfinder so it doesn't appear in the screenshot
	viewFinder.visible = false;
	
	// Get the image format set in preferences
	var imgFormat = preferences.imageFormat.value;	
	
	strFilename = saveAs(imgFormat);
	
	// Take the screenshot according to the type of image format specified
	if (imgFormat == "JPG")
	{
		var imgQuality = preferences.jpegQuality.value;
		var strCommand = "./ScreenShooter.exe -r " + "-" + imgFormat + " " + imgQuality + " " + x + "," + y + "," + width + "," + height + " \"" + resolvePath(strFilename) + "\"";	
	} else{
		var strCommand = "./ScreenShooter.exe -r " + "-" + imgFormat + " " + x + "," + y + "," + width + "," + height + " \"" + resolvePath(strFilename) + "\"";	
	}
	var result = runCommand(strCommand);
	
	// Check if the screenshot was taken successfully and print result to debugger
	if (result == "1")
	{
		// Ask if the user would like to view the screenshot
		var answer = alert("Screenshot taken successfully! Would you like to view it now?", "Yes", "No");
		
		// Check their answer and act accordingly
		if (answer == 1)
		{
			openScreenshot(strFilename);
		}
	} else{
		alert("Screenshot not taken! Error returned:\n\n" + result);
	}
}

function showViewFinder()
{
	// If the view finder is not visible then make it so
	// else do nothing
	if (viewFinder.visible == false)
		viewFinder.visible = true;
}

function captureCurrentDesktop()
{
	// The capture desktop option is nice and simple
		
	// Get the elements required to take the screenshot
	var imgFormat = preferences.imageFormat.value; // The format to save the image in
	
	var strFilename = saveAs(imgFormat);
	
	if (imgFormat == "JPG")
	{
		var imgQuality = preferences.jpegQuality.value;
		var strCommand = "./ScreenShooter.exe -s " + "-" + imgFormat + " " + imgQuality + " \"" + resolvePath(strFilename) + "\"";	
	} else{
		var strCommand = "./ScreenShooter.exe -s " + "-" + imgFormat + " \"" + resolvePath(strFilename) + "\"";	
	}
	
	var result = runCommand(strCommand);
	
	// Check if the screenshot was taken successfully and print result to debugger
	if (result == "1")
	{
		// Ask if the user would like to view the screenshot
		var answer = alert("Screenshot taken successfully! Would you like to view it now?", "Yes", "No");
		
		// Check their answer and act accordingly
		if (answer == 1)
		{
			openScreenshot(strFilename);
		}
	} else{
		alert("Screenshot not taken! Error returned:\n\n" + result);
	}
}

function openScreenshot(image)
{
	/* This function is called if the user answers yes to the question posed when a
	   screenshot is successfully taken.
	*/
	
	// Check the image exists, it should! but just in case ;-)
	if (filesystem.itemExists(image))
	{
		filesystem.open(image);
	} else{
		alert("The image doesn't seem to exist!");
	}
}